@@ -58,15 +58,20 @@ EPOCH = 1293840000
 
 def want_bytes(s, encoding='utf-8', errors='strict'):
     if isinstance(s, text_type):
-        s = s.encode(encoding, errors=errors)
+        s = s.encode(encoding, errors)
     return s
 
 
 def is_text_serializer(serializer):
-    """Checks weather a serializer generates text or binary."""
+    """Checks wheather a serializer generates text or binary."""
     return isinstance(serializer.dumps({}), text_type)
 
 
+# Starting with 3.3 the standard library has a c-implementation for
+# constant time string compares.
+_builtin_constant_time_compare = getattr(hmac, 'compare_digest', None)
+
+
 def constant_time_compare(val1, val2):
     """Returns True if the two strings are equal, False otherwise.
 
@@ -76,6 +81,8 @@ def constant_time_compare(val1, val2):
 
     This is should be implemented in C in order to get it completely right.
     """
+    if _builtin_constant_time_compare is not None:
+        return _builtin_constant_time_compare(val1, val2)
     len_eq = len(val1) == len(val2)
     if len_eq:
         result = 0
@@ -515,9 +522,7 @@ class Serializer(object):
         The return value can be either a byte or unicode string depending
         on the format of the internal serializer.
         """
-        payload = self.dump_payload(obj)
-        if isinstance(payload, text_type):
-            payload = payload.encode('utf-8')
+        payload = want_bytes(self.dump_payload(obj))
         rv = self.make_signer(salt).sign(payload)
         if self.is_text_serializer:
             rv = rv.decode('utf-8')
